package Tool;

use strict;
use vars qw/%_fields @local_aliases $localhost_name/;

# Used to translate the month's name into it's number
%input::month_numbers = (	'Jan'	=>	1,
				'Feb'	=>	2,
				'Mar'	=>	3,
				'Apr'	=>	4,
				'May'	=>	5,
				'Jun'	=>	6,
				'Jul'	=>	7,
				'Aug'	=>	8,
				'Sep'	=>	9,
				'Oct'	=>	10,
				'Nov'	=>	11,
				'Dec'	=>	12	);

%_fields = (	# the CGI fields that different search engines use to store the search strings in
		'MT'		=>	'lycos',	# hotbot.lycos.com
		'ask'		=>	'ask.com',	# ask.com/main/metaAnswer.asp
		'origq'		=>	'msn',		# search.msn.com/results.asp
		'p'		=>	'yahoo',	# google.yahoo.com/bin/query
		'q'		=>	'google|freshmeat',	# google.com/search, freshmeat.net/search, google.de/search
		'qs'		=>	'virgilio',	# search.virgilio.it/search/cgi/search.cgi
		'query'		=>	'lycos',	# search-arianna.iol.it/abin/internationalsearch, search.lycos.com/main/default.asp, suche.lycos.de/cgi-bin/pursuit
		'search'	=>	'altavista|excite'	# altavista.com/iepane, search.excite.ca/search.gw
	);

@local_aliases = (keys %{$::config->{'local'}});
if ($::config->{'referer'}{'localhost'})
{
	$localhost_name = 'localhost';
}
else
{
# Empty values will be ignored by data::save..
	$localhost_name = '';
}

sub detect_referer
# Used to extract the referer if parsing webserver
# logs.
{
	my $referer = shift;

# Remove everything after a questionmark, since
# this is data we don't need to appear in the
# statistic. (used for CGI scripts, e.g.)
	$referer =~ s/\?.*$//;
	my $local = '';
	foreach $local (@local_aliases)
	{
# match, case insensitive, everything that ENDS
# with the string defined in "local"..
		if ($referer =~ /$local/i)
		{
			$referer = $localhost_name;
		}
	}
# Remove leading characters, such as "http://www."
# or "ftp://", etc.
	$referer =~ s/^.+?:\/\/(?:www\.)?//i;
# Remove an ending slash.. This is so there are
# not "/path/to/dir" AND "/path/to/dir/" used and
# they get treated as two different things/files..
	$referer =~ s/\/$//;
# If "host_only" is set, remove port information
# and the absolute path to the file, leaving just
# the plain server name..
	if ($referer =~ /^(.+?)(?::\d+)?\/.*$/ and $::config->{'referer'}{'host_only'})
	{
		$referer = $1;
	}
	
	return $referer;
}

sub detect_browser
# This is used to (try) to translate the browser
# string into something more human-readable and
# to have a smaller number of browsers so
# information is easier to cathegorize.. If you
# don't understand this routine without comments
# you should invest in some perl book, I think..
{
	my $browser = shift;
	my $name = 'unknown';
	if ($browser =~ /Lynx/i) { $name = 'Lynx'; }
	elsif ($browser =~ /Links/i) { $name = 'Links'; }
	elsif ($browser =~ /Opera/i) { $name = 'Opera'; }
	elsif ($browser =~ /WebTV/i) { $name = 'WebTV'; }
	elsif ($browser =~ /Konqueror/i) { $name = 'Konqueror'; }
	elsif ($browser =~ /curl/i) { $name = 'curl'; }
	elsif ($browser =~ /wget/i) { $name = 'wget'; }
	elsif ($browser =~ /GetRight|GoZilla/i) { $name = 'Download Manager'; }
	elsif ($browser =~ /bot|Google|Slurp|Scooter|Spider|Infoseek|Crawl|Mercator|FireBall|av\.com/i) { $name = 'Search Engines'; }
	elsif ($browser =~ /squid/i) { $name = 'Squid Proxy'; }
# little hint: put the most used to the very end..
	elsif ($browser =~ /MSIE/i) { $name = 'MSIE'; }
	elsif ($browser =~ /Galeon/i) { $name = 'Galeon'; }
	elsif ($browser =~ /Gecko/i) { $name = 'Mozilla'; }
	elsif ($browser =~ /Mozilla.+compatible/i) { $name = 'Mozilla Compatible'; }
	elsif ($browser =~ /Nav|Gold|Mozilla|Netscape/i) { $name = 'Netscape'; }
	elsif ($::debug > 1) { print "## BROWSER $browser\n"; }
	return $name;
}

sub detect_os
# uses the same string "detect_browser" does,
# except for that it extracts the operating system
# as good as possible.
{
	my $os = $_[0];
	my $name = 'unknown';
	if ($os =~ /Mac|PPC/i) { $name = 'MacOS'; }
	elsif ($os =~ /BSD/i) { $name = 'BSD'; }
	elsif ($os =~ /Sun/i) { $name = 'SunOS'; }
	elsif ($os =~ /IRIX/i) { $name = 'IRIX'; }
	elsif ($os =~ /BeOS/i) { $name = 'BeOS'; }
	elsif ($os =~ /OS.*2/i) { $name = 'OS/2'; }
	elsif ($os =~ /AIX/i) { $name = 'AIX'; }
	elsif ($os =~ /Linux|X11|KDE|Genome|Gnome/i) { $name = 'Linux'; }
	elsif ($os =~ /ix/i) { $name = 'some UNIX'; }
        elsif ($os =~ /Win.*95/i) { $name = 'Windows 95'; }
        elsif ($os =~ /Win.*98/i) { $name = 'Windows 98'; }
        elsif ($os =~ /Win.*Me/i) { $name = 'Windows ME'; }
        elsif ($os =~ /Win.*NT/i) { $name = 'Windows NT'; }
        elsif ($os =~ /Win.*(2000|2k)/i) { $name = 'Windows 2000'; }
	elsif ($os =~ /Win/i) { $name = 'some Windows'; }
	elsif ($os =~ /Amiga/i) { $name = 'AmigaOS'; }
	elsif ($::debug > 1) { print "## SYSTEM $os\n"; }
	return $name;
}

sub extract_data
# This routine looks for data in the referer and
# extracts terms that visitors of this site were
# searching for at ome of the major searchengines.
# I know that my list is far from being complete.
# If your favorite search engine isn't included
# please feel free to contact me.
{
# If there is a field that may contain such
# information, then it's this one..
	my $referer = shift;

# We will save every field (if any) here with it's
# data being the value..
	my %form = ();
	my ($key, $val) = ('', '');

# $server is the server the visitor is coming
# from, $string the entire data which will need
# soem decoding..
	my ($server, $string) = split (/\?/, $referer, 2);

# Don't do anything unless there is any data..
# We have to return an empty list since zero would
# get interpreted as a one-element array with the
# only value being "0", making zero the top word..
	return () unless $string;

	my $field = '';
	my %words = ();

# Split data into key=value pairs
	foreach (split (/\&/, $string))
	{
		($key, $val) = split (/=/, $_, 2);
		next unless defined $val;

# A "+" in the request-string means a whitespace
		$val =~ s/\+/ /g;

# Ignore all special characters.. I know that's
# lazy and will screw up words like "foo-bar", but
# IMO it does more good than bad. If you don't
# think so either uncomment the appended line or
# write better code and drop me a copy..
#		$val =~ s/\%(.{2})/pack("C", hex($1))/eg;
		$val =~ s/\%(.{2})//g;
		$form{$key} = $val;
	}

# Print the hash's content to STDOUT if you set
# $main::debug to anything higher than 2 (3, eg.)
# This is extremely usefull for finding search-
# engines and which fields they are using..
# use './yaala | grep DATA | sort | less' for the
# best/easiest to read results..
	if ($::debug > 2)
	{
		map { print "## DATA $_: " . $form{$_} . "\n"; } (keys %form);
	}

	my $regexp;
# Cycles through every PREdefined field that may
# contain the information we want. If this field
# exists, we check wether the previous visited
# server matches the regexp (the corresponding
# value in %_fields) and if that's the case, we
# split the line into words saving it to %words to
# prevent duplicates. (otherwise a search for
# "foo foo foo foo foo foo foo" would result into
# increasing "foo" dramatically..
	foreach $field (keys %_fields)
	{
# check for this field's existance..
		next unless defined $form{$field};

		$regexp = $_fields{$field};

# check wether the server matches out regexp..
		next unless $server =~ /$regexp/i;

		$string = lc ($form{$field});

# this is a google-only thing that appears when
# the visitor used google's cache option..
		next if $string =~ /^cache:/;

# And, after all these tests, save the data..
		map { if (length ($_) > 2) { $words{$_} = 1; } } (split (/\s+/, $string));
	}
# return %words's keys as a list, which may be
# empty..
	return keys %words;
}

1;
