package input;

use strict;
use vars qw ( %month_numbers %extra );
require Tool;

print "Using Apache's access-log format <parser/apache_access.pm>\n" if $main::debug;

$extra{'total'} = 0;
$extra{'days'} = {};
$extra{'codered'} = 0;
$extra{'nimda'} = 0;

data::init (    Host		=>	'count',
                User		=>	'count',
		Date		=>	'count',
		Hour		=>	'count',
		Domain		=>	'count',
		TLD		=>	'count',
		File		=>	'count',
		Status		=>	'count',
		Bytes		=>	'byte',
		Requests	=>	'sum'		);

sub parse
{
	my $line = shift or return undef;
	if ($line =~ /^(\S+)\s(\S+)\s(.+?)\s\[(.+?)\]\s"(.+?)"\s(\d+)\s(.+)$/)
	{
		my ($host, $ident, $user, $date, $request, $status, $bytes) = ($1, $2, $3, $4, $5, $6, $7);

		my ($tmp, $lulu) = split (/\s/, $date);
		my ($day, $month, $tmp2) = split (/\//, $tmp);
		my ($year, $hour, $minute, $second) = split (/\:/, $tmp2);

		my ($method, $tmp3, $tmp4) = split (/ /, $request);
		my ($file, $params) = split (/\?/, $tmp3);

		my $attack = 0;
		if (lc ($file) eq '/default.ida') { ++$extra{'codered'}; $attack = 1; }
		elsif (lc ($file) eq '/scripts/root.exe') { ++$extra{'nimda'}; $attack = 1; }
		elsif ($file =~ /root\.exe|winnt/i) { $attack = 1; }
		if ($attack and !$::config->{'inc_attacks'}) { return 'attack'; }

		if (!($user eq '-') and ($status > 400) and ($status < 500)) { $user = 'invalid'; }

		my ($domain, $tld);
		if ($host =~ /(?:.+\.)?(.+?)\.([a-z]{2,4})$/)
		{
			$tld = lc ($2);
			$domain = lc ($1) . ".$tld";
		}
		else
		{
			$domain = 'UNRESOLVED';
			$tld = 'UNRESOLVED';
		}

		my $n_date = $year . '-' . $month_numbers{$month} . '-' . $day;

		if ($user eq '-') { $user = 'unknown'; }
		if ($bytes eq '-') { $bytes = 0; }

		++$extra{'total'};
		if (!defined $extra{'days'}->{$n_date}) { $extra{'days'}->{$n_date} = 1; }

		my %combined = (
					'Host'		=>	$host,
					'User'		=>	$user,
					'Date'		=>	$n_date,
					'Hour'		=>	$hour,
					'Domain'	=>	$domain,
					'TLD'		=>	$tld,
					'File'		=>	$file,
					'Status'	=>	$status,
					'Bytes'		=>	$bytes,
					'Requests'	=>	1
				);
		data::save (\%combined);
	}
	else { print "$line\n"; }
}

sub extra
{
	my ($average, $days) = (0, 0);
	$days = scalar keys %{$extra{'days'}};
	$average = int (($extra{'total'} * 10) / $days) / 10;
	$::extra->{'Total requests'} = $extra{'total'};
	$::extra->{'Averange requests per day'} = $average;
	$::extra->{'Reporting period'} = "$days days";
	$::extra->{'CodeRed attacks'} = $extra{'codered'};
	$::extra->{'Nimda attacks'} = $extra{'nimda'};
}

1;
